/************************************************************************
 *
 * Copyright (C) 2005 by Sistemas Embebidos S.A.
 * A Digi International Company.
 * All rights reserved.
 *
 * $Id: ns9xx0_spi.h,v 1.3 2005/09/12 10:19:33 jdietsch Exp $
 * @Author: Pedro Perez de Heredia
 * @Descr: SPI communication types and constants for the NS9750/NS9360
 *         NetSilicon uControllers.
 * @References: [1] NS9360 Hardware Reference Manual Version 90000675_A
 *                  March 2005.
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 ************************************************************************
 * @History:
 *	01/06/2005	PPH: Initial Version.
 *	12/09/2005	JD: removed inline.
 *
 ************************************************************************
 * @TODO, @TOIMPROVE
 *
 ************************************************************************/

#ifndef __NS9XX0_SPI_H
#define __NS9XX0_SPI_H

#ifdef __KERNEL__

#include <linux/config.h>
#include <linux/module.h>

#include <asm/arch/ns9750_ser.h>	// NS9750 serial registers
#include <asm/arch/ns9750_bbus.h>	// set_gpio_cfg
#include <asm/arch/hardware.h>		// BBUS_CLK_FREQ
#include <asm/arch/irqs.h>		// IRQ_BBUS_S1RXE
#include <asm/delay.h>			// udelay


#define	NS_SPI_DEFAULT_BITRATE		50000
#define	NS_SPI_MAX_BITRATE		(BBUS_CLK_FREQ / 8)

typedef enum
{
	SPI_PORTA = 0,
	SPI_PORTB,
	SPI_PORTC,
	SPI_PORTD
}spi_port_t;

typedef enum
{
	SPI_MASTER = 0,
	SPI_SLAVE
}spi_master_t;

typedef enum
{
	SPI_MODE_0 = 0,
	SPI_MODE_1,
	SPI_MODE_2,
	SPI_MODE_3
}spi_mode_t;

typedef enum
{
	SPI_LSBF = 0,
	SPI_MSBF
}spi_bitord_t;


typedef struct {
	spi_port_t	port;
	spi_master_t	master;		// master/slave
	spi_mode_t	mode;		// mode
	spi_bitord_t	bit_order;	// lsb/msb first
	int		bitrate;	// bitrate in bps
	const char	*gpios;		// list of spi gpios
	int		irq_rx;		// irq number rx
	int		irq_tx;		// irq number tx
	int		dma_tx;		// dma channel for tx
	int		dma_rx;		// dma channel for reception
	unsigned long	base_addr;	// Port base addr
	unsigned long	offset;		// Specific offset for every port
}ns_spi_dev;

// Order is SPI_TX, SPI_RX, SPI_CLK and SPI_EN
static const char ns_spi_gpios[][4] = {
	{  8,  9, 14, 15 }, /* Port A */
	{  0,  1,  6,  7 }, /* Port B */
	{ 40, 41, 22, 23 }, /* Port C */
	{ 44, 45, 26, 27 }, /* Port D */
};

void ns9xx0_spi_dump_regs( ns_spi_dev *dev );
void __init ns9xx0_spi_init_port( ns_spi_dev *dev, int port );
int ns9xx0_spi_set_bitorder( ns_spi_dev *dev, spi_bitord_t order );
int ns9xx0_spi_set_master_slave( ns_spi_dev *dev, spi_master_t master );
int ns9xx0_spi_set_mode( ns_spi_dev *dev, spi_mode_t mode );
int ns9xx0_spi_set_bitrate( ns_spi_dev *dev, int bitrate );
int ns9xx0_spi_port_setup( ns_spi_dev *dev );
int ns9xx0_spi_request_port( ns_spi_dev *dev );
void ns9xx0_spi_release_port( ns_spi_dev *dev );
void ns9xx0_spi_stop_tx_dma( ns_spi_dev *dev );
void ns9xx0_spi_start_tx_dma( ns_spi_dev *dev );
void ns9xx0_spi_stop_rx_dma( ns_spi_dev *dev );
void ns9xx0_spi_start_rx_dma( ns_spi_dev *dev );
void ns9xx0_spi_tx_byte( ns_spi_dev *dev, unsigned char byte );
void ns9xx0_spi_tx_word( ns_spi_dev *dev, unsigned short int word );
void ns9xx0_spi_tx_dword( ns_spi_dev *dev, unsigned int dword );
unsigned char ns9xx0_spi_rx_byte( ns_spi_dev *dev );
unsigned short ns9xx0_spi_rx_word( ns_spi_dev *dev );
unsigned int ns9xx0_spi_rx_dword( ns_spi_dev *dev );

#endif // __KERNEL__


// ioctl magic numbers
#define NS9XX0_SPI_IOCTL_BASE	'S'

// configs
#define NS9XX0_SPI_CONFIG_BITRATE	_IOW (NS9XX0_SPI_IOCTL_BASE, 0, int)	/* config the baudrate */
#define NS9XX0_SPI_CONFIG_MODE		_IOW (NS9XX0_SPI_IOCTL_BASE, 1, int)	/* config mode */
#define NS9XX0_SPI_CONFIG_MS		_IOW (NS9XX0_SPI_IOCTL_BASE, 2, int)	/* config MASTER/SLAVE */
#define NS9XX0_SPI_CONFIG_BITORD	_IOW (NS9XX0_SPI_IOCTL_BASE, 3, int)	/* config bit order */


// writes
//#define GPIO_CONFIG_AS_OUT	_IOR (GPIO_IOCTL_BASE, 5, int)	/* config this pin as output */
//#define GPIO_WRITE_PIN_VAL	_IOW (GPIO_IOCTL_BASE, 6, int)	/* sets the pin value */
//#define GPIO_CONFIG_PULL_UP	_IOW (GPIO_IOCTL_BASE, 7, int)	/* sets the pull-up value */

// reads
//#define GPIO_CONFIG_AS_IRQ	_IOR (GPIO_IOCTL_BASE, 10, int)	/* config this pin as interrupt */



#endif // __NS9XX0_SPI_H

